//
// CellRenderer.h
//
// $Id: //poco/Main/WebWidgets/ExtJS/include/Poco/WebWidgets/ExtJS/CellRenderer.h#2 $
//
// Library: ExtJS
// Package: Core
// Module:  CellRenderer
//
// Definition of the CellRenderer class.
//
// Copyright (c) 2007, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef ExtJS_CellRenderer_INCLUDED
#define ExtJS_CellRenderer_INCLUDED


#include "Poco/WebWidgets/ExtJS/ExtJS.h"
#include "Poco/WebWidgets/Renderer.h"


namespace Poco {
namespace WebWidgets {

	class Cell;

namespace ExtJS {


class ExtJS_API CellRenderer: public Poco::WebWidgets::Renderer
	/// CellRenderer extends Renderer with methods needed to render Cells inside tables
{
public:
	CellRenderer();
		/// Creates the CellRenderer.

	void renderBody(const Renderable* pRenderable, const RenderContext& context, std::ostream& ostr);
		/// Emits code for the page body to the given output stream.

	virtual void renderCell(const Cell* pCell, const RenderContext& context, std::ostream& ostr) = 0;
		/// Renders a cell inside a Table, the cell is initialized with a variable named var
		/// The config data inside this variable (single field or array) depends on the
		/// writeConfigData method

	virtual void writeConfigData(const Cell* pCell, const RenderContext& context, std::ostream& ostr) = 0;
	/// Writes configuration data to the outputstream (e.g.: a CheckBox needs a label and a checked state.
	/// writes -> ['label', true]

protected:
	virtual ~CellRenderer();
		/// Destroys the CellRenderer.
};


inline void CellRenderer::renderBody(const Renderable* pRenderable, const RenderContext& context, std::ostream& ostr)
{
}


} } } // namespace Poco::WebWidgets::ExtJS


#endif // ExtJS_CellRenderer_INCLUDED
