//
// Button.h
//
// $Id: //poco/Main/WebWidgets/include/Poco/WebWidgets/Button.h#8 $
//
// Library: WebWidgets
// Package: Controls
// Module:  Button
//
// Definition of the Button class.
//
// Copyright (c) 2008, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef WebWidgets_Button_INCLUDED
#define WebWidgets_Button_INCLUDED


#include "Poco/WebWidgets/Control.h"
#include "Poco/WebWidgets/Event.h"
#include "Poco/WebWidgets/AjaxDelegate.h"
#include "Poco/WebWidgets/JavaScriptEvent.h"


namespace Poco {
namespace WebWidgets {


class ButtonCell;


class WebWidgets_API Button: public Control
	/// A plain old Pushbutton.
{
public:
	typedef Poco::AutoPtr<Button> Ptr;
	
	JavaScriptEvent<AjaxParameters> buttonClicked;

	Button(const std::string& name);
		/// Creates a Button with the given name.

	Button(const std::string& name, const std::string& lblTxt);
		/// Creates a Button with the given name and lblTxt.

	Button();
		/// Creates an anonymous Button.

	void fireButtonClicked(void* pSender, AjaxParameters& params);
		/// Fires the ajaxButtonClicked event.

protected:
	Button(const std::string& name, const std::type_info& type);
		/// Creates a Button and assigns it the given name.

	Button(const std::type_info& type);
		/// Creates a Button.

	Button(const std::string& name, const std::type_info& type, Cell::Ptr ptrCell);
		/// Creates a Button and assigns it the given name.

	Button(const std::type_info& type, Cell::Ptr ptrCell);
		/// Creates a Button.

	~Button();
		/// Destroys the Button.

	void init();
		/// Common init code for all ctors.

	void init(Cell::Ptr pCell);
		/// Common init code for all ctors.
};


} } // namespace Poco::WebWidgets


#endif // WebWidgets_Button_INCLUDED
