//
// ToggleButton.h
//
// $Id: //poco/Main/WebWidgets/include/Poco/WebWidgets/ToggleButton.h#6 $
//
// Library: WebWidgets
// Package: Controls
// Module:  ToggleButton
//
// Definition of the ToggleButton class.
//
// Copyright (c) 2007, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef WebWidgets_ToggleButton_INCLUDED
#define WebWidgets_ToggleButton_INCLUDED


#include "Poco/WebWidgets/Control.h"
#include "Poco/WebWidgets/Event.h"
#include "Poco/WebWidgets/JavaScriptEvent.h"
#include "Poco/BasicEvent.h"


namespace Poco {
namespace WebWidgets {


class WebWidgets_API ToggleButton: public Control
	/// A ToggleButton is a button that can switch between two states
{
public:
	typedef Poco::AutoPtr<ToggleButton> Ptr;

	typedef Event<ToggleButton> ToggleButtonEvent;

	JavaScriptEvent<ToggleButtonEvent> checked;

	ToggleButton(const std::string& name, const std::string& label, bool isChecked);
		/// Creates a TextField with the given name.

	ToggleButton();
		/// Creates an anonymous TextField.

	void setChecked(bool checked);
		/// Sets the checked status of the object as the value of this field

	bool isChecked() const;
		/// Returns true if the object is checked

	void setLabel(const std::string& lbl);
		/// Sets the label of the ToggleButton

	const std::string& getLabel() const;
		/// Returns the label of the ToggleButton

	void fireChecked(void* pSender);
		/// Fires the checked event.

protected:
	ToggleButton(const std::string& name, const std::string& label, bool isChecked, const std::type_info& type);
		/// Creates a ToggleButton and assigns it the given name.

	ToggleButton(const std::type_info& type);
		/// Creates a ToggleButton.

	ToggleButton(const std::string& name, const std::string& label, bool isChecked, const std::type_info& type, Cell::Ptr ptrCell);
		/// Creates a ToggleButton and assigns it the given name.

	ToggleButton(const std::type_info& type, Cell::Ptr ptrCell);
		/// Creates a ToggleButton.

	~ToggleButton();
		/// Destroys the ToggleButton.

	void init(const std::string& label, bool isChecked);
		/// Common init code for all ctors.

	void init(Cell::Ptr pCell, const std::string& label, bool isChecked);
		/// Common init code for all ctors.
};


} } // namespace Poco::WebWidgets


#endif // WebWidgets_ToggleButton_INCLUDED
