//------------------------------------------------------------------------------
//	Copyright (C) 2009 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/**
 * @file TypeToString.h
 * @author Ronald Kluth
 * @date created at 2009/03/29
 * @brief Declaration and implementation of odemx::typeToString()
 * @since 3.0
 */

#ifndef ODEMX_UTIL_TYPETOSTRING_INCLUDED
#define ODEMX_UTIL_TYPETOSTRING_INCLUDED

#include <CppLog/detail/TypeToString.h>

namespace odemx {

/**
 * @brief Get a string representation of the type name
 * @param type reference to a type_info object returned by typeid()
 * @return the type name as string
 *
 * The function type_info::name() returns a mangled type name on GCC
 * and thus needs to be treated differently than on other compilers.
 * This function applies demangling to retrieve a human-readable
 * string representation of type names.
 */
inline std::string typeToString( const std::type_info& type )
{
	return Log::Detail::typeToString( type );
}

} // namespace odemx

#endif /* ODEMX_UTIL_TYPETOSTRING_INCLUDED */
